clear all;
close all;

global cscale;
global kertype;
nstart=2;
cscale=5;
%kertype='w2'; % basis function Wendland
kertype='g'; % basis function Gauss kernel
fkt='funcR2'; % data function

%load -ascii 'powerpoints1.mat' X
%load -ascii 'powerpoints2.mat' index
%load -ascii 'powerpoints3.mat' h

load powerpoints.mat X index h

maxN=65; % number of iterations length(X)

[Z1,Z2]=meshgrid(-3:h:3,-3:h:3);
Z=[Z1(:),Z2(:)];
f=evalFuncRd(fkt,Z');
len=size(Z,1);

% coefficients of Newton basis
lambda=zeros(maxN,1);
lambdaMax=zeros(maxN,1);
% coefficients of Kernel basis
alphaMax=zeros(maxN,1);

% maximum of Lebesgue function
alphaLebesgue=zeros(maxN,1);
lambdaLebesgue=zeros(maxN,1);
lagrangeLebesgue=zeros(maxN,1);

% Fill distance
hj=zeros(maxN,2);

% condition of the interpolation matrix (kernel basis)
condA=zeros(maxN,1);
% condition of the interpolation matrix (Newton basis)
condU=zeros(maxN,1);
% condition of the interpolation matrix (Lebesgue basis)
condL=ones(maxN,1);

%|| sum_j |c_j B_j(x)| ||_\infty
StandardL1=zeros(maxN,1);
NewtonL1=zeros(maxN,1);
LagrangeL1=zeros(maxN,1);

% row sum of inverse interpolation matrix
% multiplicated with the generalized Lebesgue constant 
kernelstability=zeros(maxN,1);
newtonstability=zeros(maxN,1);
lagrangestability=zeros(maxN,1);

ur=zeros(maxN,size(Z,1));
error=zeros(maxN,1);

for r=1:maxN
    r
    Y=X(1:r,:);
    fy=evalFuncRd(fkt,Y');
    
    % computation of interpolation matrices
    % on the sets X and Z
    A_xz=kernel(distsq(Y,Z));
    A_xx=kernel(distsq(Y,Y));
    condA(r)=cond(A_xx);
    
    % computation of bounds for the fill distance
    if r>1
        hj(r,:)=hbound(Y);
    end
    
    %-------- computation of u_r --------

        % via the inverse matrix
        % u=kernel(distsq(Y,Y(end,:)));
        % Ainv=invUpdate(A,Ainv,u(1:end-1,:),u(end,:));
        % U1=Ainv*A_xz;
    
    % via linear equation system
    U=A_xx\A_xz;
    ur(r,:)=U(r,:);
    condU(r)=cond(ur(1:r,index(1:r)));
    
    %-------- end computation of u_r --------
    
    % computation of the coefficients for the kernel basis
    alpha=A_xx\fy;
    
    % lebesgue function for kernel basis 
    % (largest column sum of A_xz)
    alphaLebesgue(r)=norm(A_xz,1);
    
    % lebesgue function for Newton basis
    % (largest column sum of ur)
    lambdaLebesgue(r)=norm(ur,1);
    
    % lebesgue function for Lagrange basis
    % (largest column sum of )
    lagrangeLebesgue(r)=norm(U,1);
    
    % norm of coefficients (kernel basis)
    alphaMax(r)=max(abs(alpha));

    % calculation of Power function P_r
    null=kernel(0)*ones(len,1);
    P=null-reshape(sum(U.*A_xz,1),len,1);
    
    % computation of coefficients lambda
    lambda(r)=evalFuncRd(fkt,Z(index(r),:)')-lambda'*ur(:,index(r));
    % infty-norm of coefficients vector
    lambdaMax(r)=max(abs(lambda));
    
    % values of interpolant
    fn=lambda'*ur;
    err=fn'-f;
    error(r)=max(abs(err));
    
    % computation of || sum_j |c_j B_j(x)| ||_\infty
    % for Newton basis, kernel basis and Lagrange basis
    NewtonL1(r)=max(abs(lambda')*abs(ur));
    StandardL1(r)=max(abs(alpha')*abs(A_xz));
    LagrangeL1(r)=max(abs(fy')*abs(U));
    
    % row sum of inverse interpolation matrix for Newton basis
    % multiplicated with the generalized Lebesgue constant
    newtonstability(r)=norm(inv(ur(1:r,index(1:r))),inf)*lambdaLebesgue(r);
    
    % row sum of inverse interpolation matrix for kernel basis
    % multiplicated with the generalized Lebesgue constant
    kernelstability(r)=norm(inv(A_xx),inf)*alphaLebesgue(r);
    
    % row sum of inverse interpolation matrix for Lebesgue basis
    % multiplicated with the generalized Lebesgue constant
    % row sum == 1 !!!
    lagrangestability(r)=lagrangeLebesgue(r);
    
end
r=nstart:maxN;

% function hbound needs at least two points and h_1 is
% approximately the same as h_2
% h(:,1) -> lower bound of fill distance
% h(:,2) -> upper bound of fill distance
 hj(1,:)=hj(2,:);

% error function
figure(1)
semilogy(r,error(r),'b+')
xlabel('Number of points','fontsize',12,'fontweight','b')
ylabel('Error','fontsize',12,'fontweight','b')

% data points distribution
figure(2)
plot(X(r,1),X(r,2),'r+')
xlabel('x-value data points','fontsize',12,'fontweight','b')
ylabel('y-value data points','fontsize',12,'fontweight','b')

% norm of coefficients
figure(3)
semilogy(r,alphaMax(nstart:end),'ko',r,lambdaMax(nstart:end),'k+');
xlabel('Number of points','fontsize',12,'fontweight','b')
ylabel('L_\infty norm of coefficient vector','fontsize',12,'fontweight','b')
legend('Kernel basis','Newton basis','Location','NorthWest')

% Lebesgue functionen
figure(4)
semilogy(r,alphaLebesgue(nstart:end),'ro',...
    r,lambdaLebesgue(nstart:end),'b+',...
    r,lagrangeLebesgue(nstart:end),'g*');
xlabel('Number of points','fontsize',12,'fontweight','b')
ylabel('Lebesgue function','fontsize',12,'fontweight','b')
legend('Kernel basis','Newton basis','Lagrange basis','Location','NorthWest')

% condition of interpolation matrix
figure(5)
semilogy(r,condA(nstart:end),'ko',r,condU(nstart:end),'k+');
xlabel('Number of points','fontsize',12,'fontweight','b')
ylabel('Condition number of interpolation matrix','fontsize',12,'fontweight','b')
legend('Kernel basis','Newton basis','Location','NorthWest')

% stability bound via || sum |alpha_j f_j(x)| ||_{l_\infty}
figure(6)
semilogy(r,StandardL1(nstart:end),'ko',...
    r,NewtonL1(nstart:end),'k+',r,LagrangeL1(nstart:end),'k*');
xlabel('Number of points','fontsize',12,'fontweight','b')
% ylabel('| \sum |\alpha_j f_j(x) |_{L_\infty}',...
ylabel('||\Sigma_j |\alpha_j f_j(x)| ||_{L_\infty}',...
    'fontsize',12,'fontweight','b')
legend('Kernel basis','Newton basis','Lagrange basis','Location','NorthWest')

% stability bound via S(X,w)
figure(7)
semilogy(r,kernelstability(nstart:end),'ko',...
    r,newtonstability(nstart:end),'k+',...
    r,lagrangestability(nstart:end),'k*');
xlabel('Number of points','fontsize',12,'fontweight','b')
ylabel('Stability bound S_{X,w}','fontsize',12,'fontweight','b')
legend('Kernel basis','Newton basis','Lagrange basis','Location','NorthWest')

% stability bound via S(X,w) depending on fill-distance
figure(8)
semilogy(hj(:,1),kernelstability,'ko',hj(:,1),newtonstability,'k+',hj(:,1),lagrangestability,'k*');
xlabel('Fill distance','fontsize',12,'fontweight','b')
ylabel('Stability bound: S_{X,w}','fontsize',12,'fontweight','b')
legend('Kernel basis','Newton basis','Lagrange basis','Location','NorthWest')